/* $Id: plugin.c,v 1.9 2006/12/26 16:02:44 khorben Exp $ */
/* Somewhat efficient automated firmware reverse-engineering tool
 * Coded by khorben <khorben@uberwall.org>
 * UberWall security team \../. .\../ */



#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include "plugins/plugin.h"
#include "UWfirmforce.h"


/* macros */
#define max(a, b) ((a) > (b) ? (a) : (b))


/* Plugin */
/* types */
typedef enum _PluginFlag
{
	PF_VERBOSE = 0x01
} PluginFlag;

struct _PluginHelperData
{
	PluginFlag flags;
	char const * filename;
};


Plugin * plugin_new(char const * filename)
{
	Plugin * plugin;
	PluginType * type;

	if((plugin = malloc(sizeof(*plugin))) == NULL)
	{
		UWfirmforce_error(filename, 0);
		return NULL;
	}
	if((plugin->handle = dlopen(filename, RTLD_LAZY)) == NULL)
	{
		UWfirmforce_dlerror(filename, 0);
		plugin_delete(plugin);
		return NULL;
	}
	type = dlsym(plugin->handle, "plugin_type");
	plugin->name = dlsym(plugin->handle, "plugin_name");
	plugin->magic = dlsym(plugin->handle, "plugin_magic");
	plugin->callback = dlsym(plugin->handle, "plugin_callback");
	if(type == NULL || plugin->name == NULL || plugin->magic == NULL)
	{
		UWfirmforce_dlerror(filename, 0);
		plugin_delete(plugin);
		return NULL;
	}
	plugin->type = *type;
	return plugin;
}


/* plugin_delete */
void plugin_delete(Plugin * plugin)
{
	if(plugin->handle != NULL && dlclose(plugin->handle) != 0)
		UWfirmforce_dlerror(plugin->name, 0);
	free(plugin);
}


/* accessors */
/* plugin_get_buffer_size */
size_t plugin_get_buffer_size(Plugin * plugin)
{
	unsigned int i;
	size_t len = 0;

	for(i = 0; plugin->magic[i].magic != NULL; i++)
	{
		len = max(len, plugin->magic[i].overhead);
		len = max(len, plugin->magic[i].offset
				+ plugin->magic[i].magic_len);
	}
	return len;
}


/* useful */
static int _do_printf(PluginHelper * ph, char const * format, ...);
int plugin_do(Plugin * plugin, int verbose, char const * filename, FILE * fp)
{
	static PluginHelperData data;
	PluginHelper ph;

	if(plugin->callback == NULL)
		return -1;
	data.flags = verbose ? PF_VERBOSE : 0;
	data.filename = filename;
	ph.data = &data;
	ph.printf = _do_printf;
	return plugin->callback(&ph, fp);
}

static int _do_printf(PluginHelper * ph, char const * format, ...)
{
	int ret;
	va_list arg;

	if(!(ph->data->flags & PF_VERBOSE))
		return 0;
	va_start(arg, format);
	ret = vprintf(format, arg);
	va_end(arg);
	return ret;
}


/* plugin_info */
void plugin_info(Plugin * plugin)
{
	struct
	{
		PluginType type;
		char const * description;
	} td[] =
	{
		{ PT_COMPRESSION,	"compressed"	},
		{ PT_EXECUTABLE,	"executable"	},
		{ PT_ARCHIVE,		"archive"	},
		{ PT_FORENSIC,		"forensic"	},
		{ 0,			NULL		}
	};
	unsigned int i;

	printf("%s:", plugin->name);
	for(i = 0; td[i].description != NULL; i++)
		if((td[i].type & plugin->type) == td[i].type)
			printf(" %s", td[i].description);
	fputc('\n', stdout);
}


/* plugin_match
 * RETURNS:	>= 0	matched signature id returned
 * 		else	nothing was found or an error occured */
int plugin_match(Plugin * plugin, char const * buf, size_t buf_cnt)
{
	unsigned int i;
	PluginMagic * magic;

	for(i = 0; (magic = &plugin->magic[i]) != NULL && magic->magic != NULL;
			i++)
	{
		if(buf_cnt < magic->offset + magic->magic_len
				|| memcmp(&buf[magic->offset], magic->magic,
					magic->magic_len) != 0)
			continue;
		return i;
	}
	return -1;
}
