/* $Id: plugin.h,v 1.9 2006/12/26 15:52:26 khorben Exp $ */



#ifndef UWFIRMFORCE_PLUGIN_H
# define UWFIRMFORCE_PLUGIN_H

# include <stdio.h>


/* types */
typedef enum _PluginType
{
	PT_ARCHIVE	= 0x1,
	PT_COMPRESSION	= 0x2,
	PT_FORENSIC	= 0x4,
	PT_EXECUTABLE	= 0x8
} PluginType;

typedef struct _PluginHelperData PluginHelperData;

typedef struct _PluginHelper
{
	/* private */
	PluginHelperData * data;

	/* functions */
	int (*printf)(struct _PluginHelper * ph, char const * format, ...);
} PluginHelper;

typedef struct _PluginMagic
{
	size_t overhead;
	size_t offset;
	unsigned char const * magic;
	size_t magic_len;
} PluginMagic;

typedef struct _Plugin
{
	PluginType type;
	char const * name;

	/* signature matching */
	PluginMagic * magic;
	int (*callback)(PluginHelper * ph, FILE * fp);

	/* private */
	void * handle;
} Plugin;


/* macros */
# if BYTE_ORDER == BIG_ENDIAN 
#  define htob16(a) (a) 
#  define htol16(a) ((a) << 8 | (a) >> 8)
#  define htob32(a) (a) 
#  define htol32(a) ((a) << 24 | (((a) >> 8) << 24) | (((a) << 8) >> 24) \
		| (a) >> 24)
#  define htob64(a) (a)
#  define htol64(a) ((a) << 56 | (((a) >> 8) << 48) | (((a) >> 16) << 40) \
		| (((a) >> 24) << 32) | (((a) >> 32) << 24) \
		| (((a) >> 40) << 16) | (((a) >> 48) << 8) | (a) >> 56)
# else
#  define htob16(a) ((a) << 8 | (a) >> 8)
#  define htol16(a) (a)
#  define htob32(a) ((a) << 24 | (((a) >> 8) << 24) | (((a) << 8) >> 24) \
		| (a) >> 24)
#  define htol32(a) (a) 
#  define htob64(a) ((a) << 56 | (((a) >> 8) << 48) | (((a) >> 16) << 40) \
		| (((a) >> 24) << 32) | (((a) >> 32) << 24) \
		| (((a) >> 40) << 16) | (((a) >> 48) << 8) | (a) >> 56)
#  define htol64(a) (a)
# endif


/* functions */
Plugin * plugin_new(char const * filename);
void plugin_delete(Plugin * plugin);

/* accessors */
size_t plugin_get_buffer_size(Plugin * plugin);

/* useful */
int plugin_do(Plugin * plugin, int verbose, char const * filename, FILE * fp);
void plugin_info(Plugin * plugin);
int plugin_match(Plugin * plugin, char const * buf, size_t buf_cnt);

#endif /* !UWFIRMFORCE_PLUGIN_H */
