/* $Id: tar.c,v 1.8 2006/12/26 15:52:26 khorben Exp $ */



#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "plugin.h"

#define max(a, b) ((a) > (b) ? (a) : (b))


/* types */
#pragma pack(1)
struct tar
{
	char filename[100];
	char mode[8];
	char uid[8];
	char gid[8];
	char size[12];
	char mtime[12];
	char checksum[8];
	uint8_t type;
	char link[100];
};
#pragma pack()


/* variables */
/* signatures */
static char sig[] = "ustar";

/* plugin */
PluginType plugin_type = PT_ARCHIVE;
char const plugin_name[] = "TAR";
PluginMagic plugin_magic[] =
{
	{ 512,	257,	sig,	sizeof(sig)-1	},
	{ 0,	0,	NULL,	0		}
};


/* functions */
static int _callback_filename(PluginHelper * ph, struct tar * buf);
static int _callback_mode(PluginHelper * ph, struct tar * buf);
static int _callback_uid(PluginHelper * ph, struct tar * buf);
static int _callback_gid(PluginHelper * ph, struct tar * buf);
static int _callback_mtime(PluginHelper * ph, struct tar * buf);
int plugin_callback(PluginHelper * ph, FILE * fp)
{
	struct tar buf;
	int score = 0;

	if(fread(&buf, sizeof(buf), 1, fp) != 1)
		return 0;
	score += _callback_filename(ph, &buf) / 5;
	score += _callback_mode(ph, &buf) / 5;
	score += _callback_uid(ph, &buf) / 5;
	score += _callback_gid(ph, &buf) / 5;
	score += _callback_mtime(ph, &buf) / 5;
	ph->printf(ph, "\n");
	return score;
}

static int _callback_filename(PluginHelper * ph, struct tar * buf)
{
	char tmp[sizeof(buf->filename)+1];

	memcpy(tmp, buf->filename, sizeof(buf->filename));
	tmp[sizeof(tmp)-1] = '\0';
	ph->printf(ph, "%s%s%s", "filename \"", tmp, "\"");
	return 100;
}

static int _callback_mode(PluginHelper * ph, struct tar * buf)
{
	char tmp[sizeof(buf->mode)+1];
	size_t i;
	int score = 0;

	memcpy(tmp, buf->mode, sizeof(buf->mode));
	tmp[sizeof(tmp)-1] = '\0';
	ph->printf(ph, "%s%s", ", mode ", tmp);
	for(i = 0; i < sizeof(tmp)-1; i++)
		if(tmp[i] >= '0' && tmp[i] <= '7')
			score+=14;
	return score;
}

static int _callback_uid(PluginHelper * ph, struct tar * buf)
{
	unsigned char tmp[sizeof(buf->uid)+1];
	size_t i;
	int score = 0;

	memcpy(tmp, buf->uid, sizeof(buf->uid));
	tmp[sizeof(tmp)-1] = '\0';
	ph->printf(ph, "%s%s", ", uid ", tmp);
	for(i = 0; i < sizeof(tmp)-1; i++)
		if((tmp[i] >= '0' && tmp[i] <= '7') || isalpha(tmp[i]))
			score+=12;
	return score;
}

static int _callback_gid(PluginHelper * ph, struct tar * buf)
{
	unsigned char tmp[sizeof(buf->gid)+1];
	size_t i;
	int score = 0;

	memcpy(tmp, buf->gid, sizeof(buf->gid));
	tmp[sizeof(tmp)-1] = '\0';
	ph->printf(ph, "%s%s", ", gid ", tmp);
	for(i = 0; i < sizeof(tmp)-1; i++)
		if((tmp[i] >= '0' && tmp[i] <= '7') || isalpha(tmp[i]))
			score+=12;
	return score;
}

static int _callback_mtime(PluginHelper * ph, struct tar * buf)
{
	unsigned char tmp[max(sizeof(buf->mtime)+1, 22)];
	time_t mtime;
	char * p;
	size_t i;
	int score = 0;
	struct tm tm;

	memcpy(tmp, buf->mtime, sizeof(buf->mtime));
	tmp[sizeof(tmp)-1] = '\0';
	mtime = strtol(tmp, &p, 8);
	if(*tmp == '\0' || *p != '\0')
	{
		ph->printf(ph, "%s%s", ", mtime ", tmp);
		for(i = 0; i < sizeof(tmp)-1; i++)
			if((tmp[i] >= '0' && tmp[i] <= '7') || isalpha(tmp[i]))
				score+=12;
		return score;
	}
	if((gmtime_r(&mtime, &tm)) == NULL || strftime(tmp, sizeof(tmp),
				", %d/%m/%Y %H:%M:%S", &tm) == 0)
	{
		ph->printf(ph, "%s", ", unknown date");
		return 0;
	}
	ph->printf(ph, "%s", tmp);
	return mtime < time(NULL) ? 100 : 0;
}
